;
; Wireless temperature/humidity/PIR sensor
;
; author: Niclas Forslund
;
; It supports the following senors (one of them or all at the same time is ok),
; *Dallas 1-wire temperature sensor DS18B20 and DS1822 (1-10 has been tested ok)
; *Temperature and Humidity senor DHT22/RHT03
; *PIR-sensor to detect movement. Sends data whenever a pin state change occur.
;
; The data is sent over the air with a 433MHz transmitter using the Mandolyn
; protocol every minute. This can be adjusted by changing the SAMPLE_DELAY
; parameter.
;
; Since it uses a well known protocol (ESIC and UPM use it) it will be
; compatible with the Telldus products: Tellstick DUO and Tellstick NET(?) and
; will be shown as if it was one or several ESIC/UPM temperature sensors.
; It is also likely that the RFXCom is supported since the RFXCom supports
; ESIC thermometer.
;
; House Code (HC) and Channel Code (CC)
; HC and CC for each individual 1-wire sensor is generated from every individual
; sensors unique serial ID.
; HC and CC for the DHC22/RHT03 is generated from the OSCCAL register value,
; but could be hardcoded to another value if needed.
; HC and CC for the PIR will be set by the PIR_SENSOR_ID parameter. Default
; value is 154.
;;
; Components:
;  - PIC 12F675
;  - DS18B20 Dallas Temperature sensor(s) (optional)
;  - DHT22/RHT03 Temperature/Humidity sensor (optional)
;  - PIR sensor (HC-SR501) Google "DYP-ME003/Specification.pdf"
;  - BC547B Transistor
;  - TX433 433MHz OOK transmitter (EBAY item marked FS1000A is tested OK)
;  - 2.7k resistor
;  - 2 pcs 4.7k resistor
;  - 10k resistor
;  - LED
;  - 100nF decoupling condensator
;
; Hardware description:
;      Pin 8: GND
;      Pin 7: GP0: LED voltage sensor, via 4.7k resistor to pin 5 and LED to GND
;      Pin 6: GP1: Transmitter data, connects to data TX pin on TX433
;      Pin 5: GP2: Transmitter pwr, connects to base BC547B via 2.7k resistor.
;      Pin 4: GP3: Interrupt on change. PIR-sensor data pin.
;      Pin 3: GP4: 1-wire bus to pin 2 on DS18B20 sensor, 4.7k pullup resistor to Vdd
;      Pin 2: GP5: DHT22 data pin, 10k pullup resistor to Vdd
;      Pin 1: Vdd
;
; Power supply: 3*AAA batteries, unregulated. 3*NiMH gives 3.6V
;
; 1-wire parasite mode is not supported.
;
; Transmitter protocol
;  Mandolyn
;
; Packet content for the Mandolyn protocol
;
;  - 4-bit preamble
;  - 4-bit House Code (Here, 6 byte 1-wire id XOR:ed, OSCCAL or PIR_SENSOR_ID)
;  - 2-bit Channel Code (Here, 6 byte 1-wire id XOR:ed, OSCCAL or PIR_SENSOR_ID)
;  - 2-bit Unknown (always b'11')
;  - 1-bit Battery status (here always b'0')
;  - 7-bit Humidity (here it represents battery status and/or Humidity)
;  - 12-bit Temperature from 1-wire sensors and/or DHT22
;  - 2-bit packet sequence number (0-2)
;  - 2-bit Checksum


	list            	p=12f675		; list directive to define processor
    #include            <p12f675.inc>
	errorlevel 			-302			; suppress message 302 from list file
    __CONFIG _CPD_OFF & _CP_OFF & _BODEN_OFF & _MCLRE_OFF & _PWRTE_ON & _WDT_ON & _INTRC_OSC_NOCLKOUT

;****************************************************************************
; Assign constants
;****************************************************************************
; system defines
PACKET_RESENDS          equ .3   ; Packages to send in each burst.
PIR_SENSOR_ID           equ .255 ; sensorID to use if a PIR-detect event occur.
SAMPLE_DELAY			equ	.23  ; Delay between readings, in 2.6-second units.
                                 ; 23=>1min, 115=>5min, 230=>10min


; GPIO defines
BATT_VOLT_PIN			equ GPIO0
TX_PIN					equ GPIO1
TX_POWER_PIN			equ GPIO2
IRQ_PIN                	equ GPIO3
DQ_PIN					equ GPIO4
DHT_PIN 				equ GPIO5

; DS18B20 function commands
CONVERT_T				equ	0x44
SCRATCHPAD_WRITE 		equ	0x4E
SCRATCHPAD_READ			equ	0xBE
SCRATCHPAD_COPY			equ	0x48
RECALL_EE				equ	0xB8
READ_POWER_SUPPLY 		equ	0xB4

; DS18B20 ROM commands
OwSearchRom				equ	0xF0
OwReadRom				equ	0x33
OwMatchRom				equ	0x55
OwSkipRom				equ	0xCC
OwSearchAlarm			equ	0xEC

#define	BANK1			banksel 0x80	; Select Bank1
#define	BANK0			banksel 0x00	; Select Bank0


;****************************************************************************
; Reserve Space for Variables (file registers)
;****************************************************************************
    cblock 0x20
        RomBuffer:8     ; Ow.searchRom
		DhtBuffer:5		; DHT data: RH_Int | RH_Dec | T_Int | T_Dec | CHKSUM
        LastDiscrep     ; Search
        LastZero        ; Search
        BitMask         ; Search
        BitIndex        ; Search
        OwTemp          ; Search
        OwByte
        DhtByte
        PirTemp
        BitCtr
		QHi,QLo,DDHi,DDLo
        w_temp
        status_temp
        SendBuffer
        tmp0
        delayCnt
        PacketSequenceNumber
        PacketCounter
		SensorCount
        counter
        TemperatureLo
        TemperatureHi
        Humidity
		sensorID
        CounterBitDelay
        CounterBitLongDelay
        XorParity
    endc

;--< macros >------------------------------------------------------

    ; DQ High w. strong pull-up. Only used in parasite mode
Ow.dqStrPull    macro
    bsf		GPIO, DQ_PIN					; High
    BANK1
    bcf		TRISIO, DQ_PIN					; Output
    BANK0
    endm

    ; Floating DQ High
Ow.dqHi     macro
    BANK1
    bsf		TRISIO, DQ_PIN					; Input
    BANK0
    endm

    ; Pull DQ low actively
Ow.dqLo     macro
    bcf		GPIO, DQ_PIN					; Low
    BANK1
    bcf		TRISIO, DQ_PIN					; Output
    BANK0
    endm

Dht.Hi      macro
    bsf		GPIO, DHT_PIN					; High
    endm

Dht.Lo     macro
    bcf		GPIO, DHT_PIN					; Low
    endm

    ; Delay for TIME us.
    ; TIME must be in multiples of 5.
wait    macro   time
    movlw   (time/.5)-.1
    movwf   tmp0
    call    wait5u
    endm

TxMeasure  macro
    movlw	PACKET_RESENDS
    movwf	PacketCounter
    clrf	PacketSequenceNumber
	clrwdt                       		; Clear the watchdog
    call    TxPacket
    incf	PacketSequenceNumber, f
    call    BitLongDelay				; 72ms pause between each package
    decfsz	PacketCounter, f
    goto	$-4
    endm

;****************************************************************************
; Reset Vector
;****************************************************************************
	ORG     0x00                ; processor reset vector
	goto    Init                ; Initiate the PIC and variables


    ORG     0x04                ; Interrupt Vector
    movwf   w_temp              ; Save STATUS and W register
    movf    STATUS,w
    movwf   status_temp

	btfss   GPIO, IRQ_PIN		; Read GPIO port
    goto    tidyup

    movlw   PIR_SENSOR_ID       ; Hardcoded SensorID to indicate interrupt
    movwf   sensorID

    movf    PirTemp,w
    movwf   TemperatureHi       ; Make sent package different from the previous
    incf    PirTemp,f           ; to make it compatible with Beyond Measure

    TxMeasure                   ; Transmit interrupt detection

tidyup
    bcf     INTCON, GPIF        ; Clear GPIF to clear interrupt
    movf    status_temp,w       ; Restore STATUS and W
    movwf   STATUS
    swapf   w_temp,f
    swapf   w_temp,w

    retfie

;****************************************************************************
; Main program
;****************************************************************************
MainLoop
;**************************************************************************
;  Get the temperature and humidity from the DHT22 sensor
;**************************************************************************
    bcf     INTCON, GIE             ; Disable interrupt
    call    Dht.inquire
    bsf     INTCON, GIE             ; Enable interrupt
    movf    counter,f
    skpz                            ; If no DHT22 detected, skip TxPacket
	goto	convert
	clrf	PacketSequenceNumber
    call    3FFh                    ;Get the cal value
;    movlw   .99       ; Uncomment if need for hardcoded SensorID for DHT22
    movwf   sensorID
    TxMeasure

    clrf    TemperatureLo
    clrf    TemperatureHi
    clrwdt


;**************************************************************************
;  Issue a "convert" temperature command to ALL sensors to save time
;**************************************************************************
convert
    bcf     INTCON, GIE     ; Disable interrupt while "talking" to 1wire device
    call    Ow.reset        ; reset OW devices

    movlw   OwSkipRom       ; the "skip rom" command
    call    Ow.rwByte       ;

    movlw   CONVERT_T       ; send "convert" command
    call    Ow.rwByte
    bsf     INTCON, GIE     ; Enable interrupt

    ; use 60 usec "read slots" to wait for conversion complete
rdslot
    wait    .60             ; for 60 usec 'read slot'
    call    Ow.rdBit        ; C = 1, conversion complete?
    bnc     rdslot          ; no, branch (wait)

scan
    clrf    SensorCount
    clrf    LastDiscrep                 ; start a new search for 1-wire devices
scanagn
    bcf     INTCON, GIE     ; Disable interrupt while "talking" to 1wire device
    call    OwSearchNext    ; collect a ROM ID
    bsf     INTCON, GIE     ; Enable interrupt
    movf    RomBuffer,w
    xorlw   0x28                            ; a DS18B20 device (0x28)?
    skpz                                    ; yes, skip, else
    xorlw   0x1D^0x28                       ; a DS2423b device (0x1D)?
    skpz                                    ; yes, skip, else
    goto    chklast                         ; branch (not a DS18B20 or DS2423b)

;**************************************************************************
;  Address 1w-device and send a scratchpad_read command
;**************************************************************************
    bcf     INTCON, GIE     ; Disable interrupt while "talking" to 1wire device
    call    Ow.reset        ; reset OW devices

    movlw   OwMatchRom      ; the "match rom" command
    call    Ow.rwByte       ;

    movlw   .8              ; Send the 64-bit rom-id
    movwf   counter
    movlw   RomBuffer
	movwf	FSR
nxt movf    INDF,w
    call    Ow.rwByte
    incf    FSR,f
    decfsz  counter,f
    goto    nxt

    movlw   SCRATCHPAD_READ     ; the "read scratch" command
    call    Ow.rwByte

;**************************************************************************
; Read the Scratchpad and at the same time,
; convert Temperature to Mandolyn protocol format: (temp*16+50*16).
; Since the format received from DS18B20 is already in the right format,
; I can skip the multiplication by 16 and only add 800(320h).
;**************************************************************************
    call    Ow.rdByte
    movf    OwByte, w
    addlw   0x20
	btfsc	STATUS, C
    incf	TemperatureHi, f
	movwf	TemperatureLo

    call    Ow.rdByte
    movf    OwByte, w
    addlw   0x3
    addwf   TemperatureHi, f

    call    Ow.reset
    bsf     INTCON, GIE     ; Enable interrupt


;**************************************************************************
; Generate sensorID from 1-wire-sensor serial ID
;**************************************************************************
	movlw   .6              ; XOR 6 bytes of 1wire serial number and use the
    movwf   counter			; 4 bits as House Code and 2 bits as Channel Code
    movlw   RomBuffer+.1	; from the resulting XOR:ed bytes.
	movwf	FSR
	movlw   0x00            ; Ensure a known state to start with
    xorwf	INDF,w
    incf    FSR,f
    decfsz  counter,f
    goto    $-3
	movwf	sensorID


;**************************************************************************
; Read the analog voltage
;**************************************************************************

    bcf     INTCON, GIE             ; Disable interrupt while getting batt.stat.
    bsf		GPIO, TX_POWER_PIN      ; Turn on power to LED and TX433
    BANK1
    bsf     ANSEL, ANS0				; Select analog ANS0
    BANK0
    bsf		ADCON0, ADON			; Turn on AD
    wait    .10             		; Wait 10us for analog to stabilize
    bsf		ADCON0, GO				; Start AD
	btfsc   ADCON0, GO				; Wait for AD to be ready
	goto	$-1
    bcf		ADCON0, ADON			; Turn off AD
    bcf		GPIO, TX_POWER_PIN      ; Turn off power to LED and TX433
    bsf     INTCON, GIE             ; Enable interrupt
    BANK1
    bcf		ANSEL, ANS0				; Deselect analog ANS0
    BANK0
    movf	ADRESH, w
    movwf   Humidity
    rlf     Humidity,f


;**************************************************************************
; Transmit all temperatures over TX433 one by one
;**************************************************************************
    TxMeasure
    clrwdt

    incf	SensorCount, f
    call    BitDelay                    ; 1 ms pause between each Sender
    clrf    TemperatureLo
    clrf    TemperatureHi

chklast
    movf    LastDiscrep,F               ; last device (Z = 1)?
    skpz                                ; yes, skip, else
    goto    scanagn                     ; branch and scan again


	; Sleep
	clrwdt                              ; Clear the watchdog
	movlw	SAMPLE_DELAY
	movwf	delayCnt
	sleep
	nop
 	decfsz	delayCnt, f
	goto	$-3

	goto	MainLoop                    ; Start next round...



;****************************************************************************
; Init
;****************************************************************************
Init
	clrf	PacketSequenceNumber
    clrf    SensorCount
    clrf    TemperatureLo
    clrf    TemperatureHi
    clrf    PirTemp

	BANK1
	clrf	VRCON			; VREN=0, VRR=0, VR3=0, VR2=0, VR1=0, VR0=0
							; (power off the comparator voltage, saves power)

    call    3FFh            ; Get the cal value
    movwf   OSCCAL          ; Calibrate 4MHz oscillator

    bsf     IOC, IOC3       ; Enable Interrupt on change on GPIO-pin 3
    bsf     INTCON, GPIE    ; Enable GPIO-port interrupts

;OptionBits  equ b'10001111'
 	;             ||||||||---- PS0    - Prescalar watchdog 1:128
	;             |||||||----- PS1
	;             ||||||------ PS2
	;             |||||------- PSA    - Assign prescaler to watchdog
	;             ||||-------- TOSE   - LtoH edge
	;             |||--------- TOCS   - Timer0 uses IntClk
	;             ||---------- INTEDG - falling edge RB0
	;             |----------- GPPU   - pull-ups disabled
	
;****************************************************************************
; Recommended sequence to change prescaler from TMR0 -> WDT
; without causing accidental RESET
; http://ww1.microchip.com/downloads/en/DeviceDoc/31011a.pdf
;****************************************************************************
    BANK0
    clrf    TMR0                ; Clear TMR0 and prescaler
    BANK1
    movlw   b'10001111' | .7    ; Select WDT, do not change prescale value
    movwf   OPTION_REG      	;
    clrwdt                  	; Clears WDT and prescaler
    movlw   b'10001111'
    movwf   OPTION_REG
    clrwdt
;****************************************************************************

	movlw	0x51				; Fosc/16, only AN0 is analog
	movwf	ANSEL

	BANK0
	bcf		ADCON0, ADFM		; AD left justified. 8-bit resolution (ADRESH)
	bcf		ADCON0, VCFG		; Use Vdd as a reference

	movlw	0x07				; Comparator Off
	movwf	CMCON				; for lowest Power consumption

	bcf		GPIO, BATT_VOLT_PIN
	bcf		GPIO, TX_PIN
	bcf		GPIO, TX_POWER_PIN
    bsf     GPIO, IRQ_PIN
	bsf		GPIO, DHT_PIN
	bsf		GPIO, DQ_PIN
	BANK1
	bsf		TRISIO, BATT_VOLT_PIN			; Input
	bcf		TRISIO, TX_PIN					; Output
	bcf		TRISIO, TX_POWER_PIN			; Output
	bsf		TRISIO, IRQ_PIN     			; Input
	bsf		TRISIO, DHT_PIN                 ; Input
	bcf		TRISIO, DQ_PIN					; Output
	BANK0

    Dht.Hi

	goto	convert


;****************************************************************************
; RF433 Tx functions
;****************************************************************************

TxPacket
    bsf		GPIO, TX_POWER_PIN              ; Turn on power to LED and TX433
    bcf		GPIO, TX_PIN                    ; Set TX-pin to a known state

	movlw	b'1100' 						; 4-bit Preamble
	movwf	SendBuffer
    swapf   SendBuffer,f                    ; waste 4 first bits
	call 	TxContent4bit

											; Start of chksum calculation.
    movlw   b'000'                          ; Preamble not included in chksum.
	;         |||--------- Stores XOR result of all EVEN bits
	;         ||---------- Stores XOR result of all ODD bits
	;         |----------- Last sent bit ODD/EVEN bit indicator. 1=Odd, 0=Even
    movwf   XorParity

	movf	sensorID,w
	movwf	SendBuffer
	call	TxContent6bit       ; Send both generated House Code (4-bits)
                                ; and generated Channel Code (2-bits)

	movlw	0xC0            				; 2-bit Constant
	movwf	SendBuffer
	call	TxContent2bit

    clrf	SendBuffer                      ; 1-bit Battery status
	call	TxContentBit                    ; Here it is not used

;	================================================
;	The next byte is usually used for Humidity value.
;	Here I use it to present the current battery status
;	since I do not have a humidity sensor.
;	This data slot can be used to whatever data.
;	================================================

    movf    Humidity,w
	movwf	SendBuffer
	call	TxContent7bit

;	================================================

	movf	TemperatureHi, w 				; 12-bit temperature total
	movwf	SendBuffer
	swapf   SendBuffer,f                    ; waste 4 first bits
	call	TxContent4bit
	movf	TemperatureLo, w
	movwf	SendBuffer
	call	TxContent8bit

	movf	PacketSequenceNumber, w			; 2-bit packet sequence number
	movwf	SendBuffer
    swapf   SendBuffer,f                    ; waste 6 first bits
    rlf     SendBuffer,f
    rlf     SendBuffer,f
	call	TxContent2bit

    movf    XorParity, w                    ; 2-bit parity xor
	movwf	SendBuffer
    swapf   SendBuffer,f                    ; waste 6 first bits
    rlf     SendBuffer,f
    rlf     SendBuffer,f
	call	TxContent2bit

    call    TxStop                          ; 1-bit Stop bit 600us

    bcf		GPIO, TX_PIN
    bcf		GPIO, TX_POWER_PIN              ; Turn off power to LED and TX433
	return

	; Send out multiple bits of content from 'SendBuffer'
TxContent8bit
	call	TxContentBit
TxContent7bit
	call	TxContentBit
TxContent6bit
	call	TxContentBit
TxContent5bit
	call	TxContentBit
TxContent4bit
	call	TxContentBit
TxContent3bit
	call	TxContentBit
TxContent2bit
	call	TxContentBit
	call	TxContentBit
	return

	; Send out one bit of content, left-shifted out of 'SendBuffer'
TxContentBit
	rlf		SendBuffer, f
	btfsc	STATUS, C
	goto	TxOne
	goto	TxZero

	; Send a '0' bit
TxZero
    movlw   b'100'           ; Odd/Even indicator bit bitflip is done.
    xorwf   XorParity, f     ; As a zero is being sent it does not affect parity
	btfss	GPIO, TX_PIN
    goto    ZL2H
    goto    ZH2L
ZL2H
	bsf		GPIO, TX_PIN
	call 	BitDelay
	call 	BitDelay
	return
ZH2L
	bcf		GPIO, TX_PIN
	call 	BitDelay
	call 	BitDelay
	return

	; Send a '1' bit
TxOne
    movlw   b'101'              ; Assume bit 3 is set -> prev. odd -> b'101'
    btfss   XorParity, 2        ; Is it an even number? if not -> 0x6
    movlw   b'110'
    xorwf   XorParity, f
	btfss   GPIO, TX_PIN
    goto    OL2H
    goto    OH2L
OL2H
    bsf		GPIO, TX_PIN
	call 	BitDelay
	bcf		GPIO, TX_PIN
	call 	BitDelay
	return
OH2L
    bcf		GPIO, TX_PIN
	call 	BitDelay
	bsf		GPIO, TX_PIN
	call 	BitDelay
    return

	; Send a Stop-bit. The stop-bit has a total bit length of 1.2ms
TxStop
	btfss   GPIO, TX_PIN
    goto    SL2H
    goto    SH2L
SL2H
    bsf		GPIO, TX_PIN
	call 	BitShortDelay
	bcf		GPIO, TX_PIN
	call 	BitShortDelay
	return
SH2L
    bcf		GPIO, TX_PIN
	call 	BitShortDelay
    bsf		GPIO, TX_PIN
	call 	BitShortDelay
    return

BitDelay
	movlw	.197                    ; 197 -> 1ms half wave pulse
	movwf	CounterBitDelay         ; with a total 72.1ms package length
	goto    $+1
	decfsz	CounterBitDelay, f
	goto	$-2
	return

BitLongDelay						; Used to pause between data transmissions
	movlw	.72                     ; 72ms measured on the real ESIC
	movwf	CounterBitLongDelay
    call    BitDelay
	decfsz	CounterBitLongDelay, f
	goto	$-2
	return

BitShortDelay						; Used when sending the stop-bit.
    movlw   .150                    ; 600us
	movwf	CounterBitDelay
	nop
	decfsz	CounterBitDelay, f
	goto	$-2
	return


;**************************************************************************
; One wire Search Rom
;**************************************************************************
OwSearch
    clrf    LastDiscrep     ; init Last Discrepency var'
OwSearchNext
    clrf    BitIndex        ; BitIndex = 0
    clrf    LastZero        ; LastZero = 0
    movlw   b'00000001'     ;
    movwf   BitMask         ; setup rom array bit mask
    call    Ow.reset        ; reset all 1-wire devices
    movlw   OwSearchRom
    call    Ow.rwByte       ; send "search rom" command
    movlw   RomBuffer       ;
    movwf   FSR             ; FSR = &RomBuffer[0]
GetBits
    call    Ow.rdBit        ; read bit A (in Carry)
    rlf     OwTemp,F        ; F = -------A
    call    Ow.rdBit        ; read bit B (compliment)
    rlf     OwTemp,W        ; W = ------AB
    rrf     OwTemp,F        ; put 'A' bit back into Carry
    andlw   0x03            ; AB = 00, a discrepency?
    bnz     SendRomBit      ; no, branch, use 'A' bit, else
Discrepency
    movf    BitIndex,W      ; 0..63
    subwf   LastDiscrep,W   ; 0..63
    bnc     IndexGTLD       ; index > last discrepency, use '0'
    bz      SendRomBit      ; index = last discrepency, use '1'
IndexLTLD                       ; index < last discrepency, use ROM
    movf    BitMask,W       ; get ROM bit
    andwf   INDF,W          ; is ROM bit '0'?
    bnz     SendRomBit      ; no, branch with C = 1, else
    clrc                    ; set C = 0 and
IndexGTLD
    movf    BitIndex,W      ; update 'LastZero'
    movwf   LastZero        ;
SendRomBit
    movf    BitMask,W       ; set ROM bit to value in Carry
    iorwf   INDF,F          ; set ROM bit (unconditionally)
    skpc                    ; is bit a '1'? yes, skip, else
    xorwf   INDF,F          ; clr ROM bit
    call    Ow.rwBit        ; send direction/qualifier bit
    rlf     BitMask,W       ;
    rlf     BitMask,F       ; increment rom array bit mask
    skpnc                   ; byte boundary? no, skip, else
    incf    FSR,F           ; bump RomBuffer array address
    incf    BitIndex,F      ; bump bit index, 0..63
    btfss   BitIndex,6      ; BitIndex = 64?  yes, skip, else
    goto    GetBits         ; do next bit
    movf    LastZero,W      ; update last discrepency
    movwf   LastDiscrep     ; a 0 value indicates end-of-list
    return


;**************************************************************************
; Ow.rdByte, result in 'OwByte'.
; Ow.rwByte, send byte in WREG.
;**************************************************************************
Ow.rdByte
    movlw   .8
    movwf   BitCtr
rdloop
	call	Ow.rdBit
    rrf     OwByte, f
    decfsz  BitCtr,F        ; Last bit?
    goto    rdloop          ; no, loop, else
    return                  ; exit

Ow.rwByte
    movwf   OwByte
    movlw   .8
    movwf   BitCtr
	rrf		OwByte, f
rwloop
	call	Ow.rwBit
	rrf		OwByte, f
    decfsz  BitCtr,F        ; Last bit?
    goto    rwloop          ; no, loop, else
    return                  ; exit


;**************************************************************************
;  Ow.rdBit, read bit from 1wire-bus and return result in Carry.
;  Ow.rwBit, send bit in Carry.
;**************************************************************************
Ow.rdBit
    clrc
	Ow.dqLo
	Ow.dqHi
	goto	$+1
	goto	$+1
	goto	$+1
	goto	$+1
    btfsc   GPIO,DQ_PIN     ; sample owpin at exactly 14 us
    setc                    ; clear Carry if '0'
	wait    .60
    return

Ow.rwBit
	btfsc	STATUS, C
	goto	OnewireSlotWrite1
	goto	OnewireSlotWrite0

OnewireSlotWrite0
	Ow.dqLo
	wait    .60
    Ow.dqHi
	return

OnewireSlotWrite1
	Ow.dqLo
	Ow.dqHi
	wait    .60
    return


;***************************************************************************
;  Dht.inquire: Read 6 byte from DHT-sensor and return result in DhtBuffer.
;***************************************************************************
Dht.inquire
	BANK1
	bcf		TRISIO, DHT_PIN     ; Set port to Output
	BANK0
    Dht.Lo
    wait    .1000
    wait    .1000
	Dht.Hi
    wait    .20
	BANK1
    bsf		TRISIO,DHT_PIN		; Set port to Input
	BANK0
	movlw   .5                  ; Read 5 bytes from DHT22
    movwf   counter
    movlw   DhtBuffer           ;
    movwf   FSR                 ; FSR = &DhtBuffer[0]
    wait    .25                 ;
	btfsc   GPIO, DHT_PIN		; Detect DHT22. If not, return to scan 1wire-bus
    goto    DhtEnd
	wait	.110
	btfsc   GPIO, DHT_PIN		; Wait for data ready to read
	goto	$-1
NextDhtByte
    movlw   .8
    movwf   BitCtr
    wait    .10                 ; Relax
DhtTxPrep
    clrc
	btfss   GPIO, DHT_PIN       ; Wait data ready to read
	goto	$-1
	wait	.55					; At this line we are at 55us end
	btfsc   GPIO, DHT_PIN		; DHT high, set (C)arry, else
    goto	one
zro	rlf		INDF,f
    decfsz  BitCtr,f            ; Last bit?
	goto	DhtTxPrep
	incf	FSR,f
    decfsz  counter,f           ; Last byte?
	goto	NextDhtByte
	goto	DhtPrep

one setc
    btfsc   GPIO, DHT_PIN		; Wait for data ready to read
	goto	$-1
    goto    zro

DhtPrep
    movf    DhtBuffer+.0, w
    movwf   DDHi
    movf    DhtBuffer+.1, w
    movwf   DDLo
    call    devideBy10
    movf    QLo, w
    movwf   Humidity

    movf    DhtBuffer+.2, w
    movwf   DDHi
    movf    DhtBuffer+.3, w
    movwf   DDLo
    call    devideBy10

    movlw   0x0F
    andwf   QHi,f
    clrc

    movlw   .4
    movwf   counter
    rlf     QLo,f
    rlf     QHi,f
    decfsz  counter,f
    goto    $-3

    movf    DDLo,w
    addwf   QLo,w

    addlw   0x20
	btfsc	STATUS,C
    incf	TemperatureHi,f
	movwf	TemperatureLo

    movf    QHi,w
    addlw   0x3
    addwf   TemperatureHi,f

DhtEnd
	wait	.20
	Dht.Hi

	return


;**************************************************************************
;  Ow.reset returns 1 in Carry if a presence detect pulse is found
;**************************************************************************	
Ow.reset
    clrc
    Ow.dqLo             
    wait    .500        ; Set DQ low for 500 us
    Ow.dqHi             ; Set DQ high again
    wait    .70         ; and wait 70us
    btfsc   GPIO,DQ_PIN ; sample dq_pin
    setc                ; Set Carry if '1'
    wait    .430        ; delay 430us
    return


;This takes 5us to complete
wait5u
    goto    $+1
    decfsz  tmp0,f
    goto    wait5u
    movlw   .0
    return


;**************************************************************************
; Subroutine to divide by 10 by repeated subtraction of 10.
; Arg: 16-bit dividend in DDHi, DDLo.
; Ret: 16-bit quotient in QHi, QLo, remainder in DDLo
;**************************************************************************
devideBy10
    clrf	QHi     ; Clear quotient registers
	clrf	QLo     ;
	movlw	.10     ; W = divisor
step
	subwf	DDLo,F	; Subtract 10 from dividend LSB
	btfsc	STATUS,C	; Carry?
	goto	bump	; No. Jump
	movf	DDHi,F	; Yes. Is dividend MSB 0?
	btfsc	STATUS,Z	;
	goto	done	; Yes. Done
	decf	DDHi,F	; No. Decrement dividend MSB
bump
	incf	QLo,F	; Bump quotient LSB
	btfsc	STATUS,Z	; Is it 0?
	incf	QHi,F	; Yes. Increment quotient MSB
	goto	step	; Loop until dividend is <0
done
    addwf	DDLo,F	; Correct the dividend LSB
	return

    end